#include <display_syscalls.h>
#define BLACK 0x0000
#define WHITE 0xffff
#define RGB_COLOR(r,v,b) ((b%32) + ((v%64) << 6) + ((r%32) << 11))
#define INVERSE_RGB_COLOR(r,v,b) (((31-b)%32) + (((31-v)%64) << 6) + (((31-r)%32) << 11))

#define numBlobs 10
#define width 384
#define height 216

void FillVRAM(short color)
{
	int i, color32bits;
	short* VRAM = (short*)0xA8000000;
	color32bits = (color<<16) | color;
	for(i=82944 ; i ; i--)
		*(VRAM++) = color32bits;
}

int key_down(int basic_keycode)
{
	const unsigned short* keyboard_register = (unsigned short*)0xA44B0000;
	int row, col, word, bit;
	row = basic_keycode%10;
	col = basic_keycode/10-1;
	word = row>>1;
	bit = col + ((row&1)<<3);
	return (0 != (keyboard_register[word] & 1<<bit));
}

static unsigned int lastrandom = 0x12345678;

int time_getTicks()
{
	asm("mov.l syscall_adress, r2\n"
	    "mov.l getTicks, r0\n"
	    "jmp @r2\n"
	    "nop\n"
	    "syscall_adress: .long 0x80020070\n"
	    "getTicks: .long 0x02C1");
}

void srand(unsigned int seed)
{
	lastrandom = seed;
}

int rand(void)
{
	lastrandom = 0x41C64E6D*lastrandom + 0x3039;
	return lastrandom >> 16;
}

void Pixel(unsigned short x, unsigned short y, unsigned short color)
{
    __asm__("mov.w	.width,r1\n"    
            "mulu.w	r1,r5\n"
            "mov.l	.vram,r1\n"
            "sts	macl,r0\n"
            "add	r4,r0\n"
            "shll	r0\n"
            "mov.w	r6,@(r0,r1)\n"
            "rts\n"	
            "clrmac\n"
            ".align 1\n"
            ".width:\n"
            "    .short 384\n"
            " 	 .align 2\n" 
            ".vram:\n"
            "    .long -1476395008\n"
            "	 .align 1\n");
}

void main(void) {

	srand(time_getTicks());
	int blogPx[numBlobs];
	int blogPy[numBlobs];
	// Movement vector for each blob
	int blogDx[numBlobs];
	int blogDy[numBlobs];
	int vx[numBlobs][width];
	int vy[numBlobs][height];
	int i,j,k;
	int m;
	int rouge = rand()%32, vert = rand()%32, bleu = rand()%32;
	short color, anti_color;
	
	for(i=0; i<numBlobs; i++)
	{
		blogPx[i] = rand()%width;
		blogPy[i] = rand()%height;
		if(rand()%2)
			blogDx[i] = 1;
		else
			blogDx[i] = -1;
			
		if(rand()%2)
			blogDy[i] = 1;
		else
			blogDy[i] = -1;
	}
	color = RGB_COLOR(rouge, vert, bleu);
	anti_color = INVERSE_RGB_COLOR(rouge, vert, bleu);
	
	while(!key_down(47))
	{
		if (key_down(31))
		{
			rouge = rand()%32;
			vert = rand()%32;
			bleu = rand()%32;
			color = RGB_COLOR(rouge, vert, bleu);
			anti_color = INVERSE_RGB_COLOR(rouge, vert, bleu);
		}
		
		Bdisp_AllClr_VRAM();
		for(i=0;i<numBlobs;i++)
		{
		    blogPx[i]+=blogDx[i];
			blogPy[i]+=blogDy[i];

			// bounce across screen
			if (blogPx[i] < 0 || blogPx[i] > width -1)
				blogDx[i] *= -1;
			if (blogPy[i] < 0 || blogPy[i] > height -1)
				blogDy[i] *= -1;
			for ( j = 0; j < width; j++) {
			  vx[i][j] = (blogPx[i]-j)*(blogPx[i]-j);
			}

			for ( j = 0; j < height; j++) {
			  vy[i][j] = (blogPy[i]-j)*(blogPy[i]-j);
			}
		}
		FillVRAM(anti_color);
	  for ( j = 0; j < height; j++) {
		for ( i = 0; i < width; i++) {
		  m = 1;
		  for ( k = 0; k < numBlobs; k++) {
			// Increase this number to make your blobs bigger
			m += 50000/(vy[k][j] + vx[k][i]+1);
		  }
		 if(m > 100)
			Pixel(i,j,color);
		}
	  }
	  for(i=0;i<numBlobs;i++)
		Pixel(blogPx[i],blogPy[i],anti_color);
		
		Bdisp_PutDisp_DD();
	}

	return;
}

